==> larvaworld-0.0.221-py3-none-any.whl: larvaworld/lib/model/modules/locomotor.py <==


from ... import reg
from ...param import PhaseRange, Phase, NestedConf, ClassAttr
from . import DefaultCoupling,Intermitter,Feeder,Effector

__all__ = [
    'Locomotor',
    'DefaultLocomotor',
]


class Locomotor(NestedConf):
    interference = ClassAttr(class_=DefaultCoupling, default=None, doc='The crawl-bend coupling module')
    intermitter = ClassAttr(class_=Intermitter, default=None, doc='The behavioral intermittency module')
    feeder = ClassAttr(class_=Feeder, default=None, doc='The feeding module')
    turner = ClassAttr(class_=Effector, default=None, doc='The body-bending module')
    crawler = ClassAttr(class_=Effector, default=None, doc='The peristaltic crawling module')

    def __init__(self, dt=0.1, **kwargs):
        super().__init__(**kwargs)
        # self.crawler, self.turner, self.feeder, self.intermitter, self.interference = [None] * 5
        self.dt = dt

    def on_new_pause(self):
        if self.crawler:
            self.crawler.stop_effector()
        if self.feeder:
            self.feeder.stop_effector()

    def on_new_run(self):
        if self.crawler:
            self.crawler.start_effector()
        if self.feeder:
            self.feeder.stop_effector()

    def on_new_feed(self):
        if self.crawler:
            self.crawler.stop_effector()
        if self.feeder:
            self.feeder.start_effector()

    def step_intermitter(self, **kwargs):
        if self.intermitter:
            pre_state = self.intermitter.cur_state
            cur_state = self.intermitter.step(**kwargs)
            if pre_state != 'pause' and cur_state == 'pause':
                self.on_new_pause()
            elif pre_state != 'exec' and cur_state == 'exec':
                self.on_new_run()
            elif pre_state != 'feed' and cur_state == 'feed':
                self.on_new_feed()
            # print(cur_state)


class DefaultLocomotor(Locomotor):
    def __init__(self, conf, **kwargs):
        super().__init__()
        D = reg.model.dict.model.m
        for k in ['crawler', 'turner', 'interference', 'feeder', 'intermitter']:

            if conf.modules[k]:

                m = conf[f'{k}_params']
                if k == 'feeder':
                    mode = 'default'
                else:
                    mode = m.mode
                kws = {kw: getattr(self, kw) for kw in D[k].kwargs.keys()}
                func = D[k].mode[mode].class_func
                mm = {k: m[k] for k in m.keys() if k != 'mode'}
                M = func(**mm, **kws)
            else:
                M = None
            setattr(self, k, M)

    @property
    def stride_completed(self):
        if self.crawler:
            return self.crawler.complete_iteration
        else:
            return False

    @property
    def feed_motion(self):
        if self.feeder:
            return self.feeder.complete_iteration
        else:
            return False

    def step(self, A_in=0, length=1, on_food=False):
        C, F, T, If = self.crawler, self.feeder, self.turner, self.interference
        if If:
            If.cur_attenuation = 1
        if F:
            F.step()
            if F.active and If:
                If.check_feeder(F)
        if C:
            lin = C.step() * length
            if C.active and If:
                If.check_crawler(C)
        else:
            lin = 0
        self.step_intermitter(stride_completed=self.stride_completed, feed_motion=self.feed_motion, on_food=on_food)

        if T:
            if If:
                cur_att_in, cur_att_out = If.apply_attenuation(If.cur_attenuation)
                # cur_att_in, cur_att_out = If.step(crawler=C, feeder=F)
            else:
                cur_att_in, cur_att_out = 1, 1
            ang = T.step(A_in=A_in * cur_att_in) * cur_att_out
        else:
            ang = 0

        return lin, ang, self.feed_motion

==> larvaworld-0.0.254-py3-none-any.whl: larvaworld/lib/model/modules/locomotor.py <==
import param

from ... import reg, aux
from .. import modules
from ...param import PhaseRange, Phase, NestedConf, ClassAttr, ModeSelector
from . import Coupling, Intermitter, Feeder, Effector,Crawler, Turner

__all__ = [
    'Locomotor',
    'DefaultLocomotor',
]


class Locomotor(NestedConf):
    interference = ClassAttr(class_=Coupling, default=None, doc='The crawl-bend coupling module')
    intermitter = ClassAttr(class_=Intermitter, default=None, doc='The behavioral intermittency module')
    feeder = ClassAttr(class_=Feeder, default=None, doc='The feeding module')
    turner = ClassAttr(class_=Turner, default=None, doc='The body-bending module')
    crawler = ClassAttr(class_=Crawler, default=None, doc='The peristaltic crawling module')

    def __init__(self, **kwargs):



        super().__init__(**kwargs)
        # self.crawler, self.turner, self.feeder, self.intermitter, self.interference = [None] * 5


    def on_new_pause(self):
        if self.crawler:
            self.crawler.stop_effector()
        if self.feeder:
            self.feeder.stop_effector()

    def on_new_run(self):
        if self.crawler:
            self.crawler.start_effector()
        if self.feeder:
            self.feeder.stop_effector()

    def on_new_feed(self):
        if self.crawler:
            self.crawler.stop_effector()
        if self.feeder:
            self.feeder.start_effector()

    def step_intermitter(self, **kwargs):
        if self.intermitter:
            pre_state = self.intermitter.cur_state
            cur_state = self.intermitter.step(**kwargs)
            if pre_state != 'pause' and cur_state == 'pause':
                self.on_new_pause()
            elif pre_state != 'exec' and cur_state == 'exec':
                self.on_new_run()
            elif pre_state != 'feed' and cur_state == 'feed':
                self.on_new_feed()
            # print(cur_state)

    @property
    def stride_completed(self):
        if self.crawler:
            return self.crawler.complete_iteration
        else:
            return False

    @property
    def feed_motion(self):
        if self.feeder:
            return self.feeder.complete_iteration
        else:
            return False


class DefaultLocomotor(Locomotor):
    def __init__(self,conf,dt=0.1,  **kwargs):
        self.dt = dt
        # kwargs = aux.AttrDict(kwargs)



        # D = reg.model.dict.model.m
        for k in ['crawler', 'turner', 'interference', 'feeder', 'intermitter']:

            if conf.modules[k]:

                m = conf[f'{k}_params']
                if k != 'interference':
                    m['dt'] = dt
                if k == 'feeder':
                    kwargs[k] = self.param[k].class_(**m)
                else:
                    mode = m.mode
                    mm = {k: m[k] for k in m.keys() if k != 'mode'}
                    kwargs[k] = self.param[k].class_.select(mode)(**mm)
                    # kws = {kw: getattr(self, kw) for kw in D[k].kwargs.keys()}
                    # func = D[k].mode[mode].class_func
                    # kwargs[k] = func(**mm, **kws)
            # else:
            #     M = None
            # setattr(self, k, M)
        super().__init__(**kwargs)



    def step(self, A_in=0, length=1, on_food=False):
        C, F, T, If = self.crawler, self.feeder, self.turner, self.interference
        if If:
            If.cur_attenuation = 1
        if F:
            F.step()
            if F.active and If:
                If.check_feeder(F)
        if C:
            lin = C.step() * length
            if C.active and If:
                If.check_crawler(C)
        else:
            lin = 0
        self.step_intermitter(stride_completed=self.stride_completed, feed_motion=self.feed_motion, on_food=on_food)

        if T:
            if If:
                cur_att_in, cur_att_out = If.apply_attenuation(If.cur_attenuation)
                # cur_att_in, cur_att_out = If.step(crawler=C, feeder=F)
            else:
                cur_att_in, cur_att_out = 1, 1
            ang = T.step(A_in=A_in * cur_att_in) * cur_att_out
        else:
            ang = 0

        return lin, ang, self.feed_motion
